# Somewhat useless but good to distinguish the difference, perhaps in the
# future Python's typing system will have something similar to TypeScript's
# "Template Literal Types" which is able to describe a snowflake.
# https://discord.com/developers/docs/reference#snowflakes
Snowflake = str
